@@ -1,12 +1,12 @@
 package Muster::Hook::DynamicField;
-$Muster::Hook::DynamicField::VERSION = '0.62';
+$Muster::Hook::DynamicField::VERSION = '0.92';
 =head1 NAME
 
 Muster::Hook::DynamicField - Muster hook for dynamic fields
 
 =head1 VERSION
 
-version 0.62
+version 0.92
 
 =head1 SYNOPSIS
 
@@ -29,7 +29,6 @@ use Muster::LeafFile;
 use YAML::Any;
 use POSIX qw(strftime);
 use Math::Calc::Parser;
-use Muster::Hook::Costings;
 
 =head1 METHODS
 
@@ -71,17 +70,12 @@ sub process {
     my $leaf = $args{leaf};
     my $phase = $args{phase};
 
-    if (!$leaf->is_page)
-    {
-        return $leaf;
-    }
     if ($phase ne $Muster::Hooks::PHASE_BUILD)
     {
         return $leaf;
     }
 
     my $content = $leaf->cooked();
-    my $page = $leaf->pagename;
 
     # substitute {{!var}} variables
     $content =~ s/(\\?)\{\{\!([-\w]+)\}\}/$self->get_dynamic_value($1,$2,$leaf)/eg;
@@ -175,21 +169,6 @@ sub get_function_result {
         my $result = Math::Calc::Parser->evaluate($argvals);
         $value = "${argvals} = ${result}";
     }
-    elsif ($func eq 'dyncost')
-    {
-        # dyncost(per_hour)
-        if ($leaf->{meta}->{labour_time}
-                or $leaf->{meta}->{materials_cost})
-        {
-            my $cost_per_hour = $argvals;
-            my $labour_cost = ($leaf->{meta}->{labour_time} / 60) * $cost_per_hour;
-            my $itemize_cost = ($leaf->{meta}->{itemize_time} / 60) * $cost_per_hour;
-            my $wholesale = $leaf->{meta}->{materials_cost} + $labour_cost + $itemize_cost;
-            my $overheads = Muster::Hook::Costings::calculate_overheads($wholesale);
-            my $retail = $wholesale + $overheads;
-            $value = "dyncost($cost_per_hour) = $retail";
-        }
-    }
 
     if (!defined $value)
     {
