@@ -1,6 +1,6 @@
 package Muster::Hook::HeadFoot;
-$Muster::Hook::HeadFoot::VERSION = '0.62';
-use Mojo::Base 'Muster::Hook::Directives';
+$Muster::Hook::HeadFoot::VERSION = '0.92';
+use Mojo::Base 'Muster::Hook';
 use Muster::LeafFile;
 use Muster::Hooks;
 
@@ -14,7 +14,7 @@ Muster::Hook::HeadFoot - Muster header and footer hook.
 
 =head1 VERSION
 
-version 0.62
+version 0.92
 
 =head1 DESCRIPTION
 
@@ -22,8 +22,6 @@ L<Muster::Hook::HeadFoot> includes header and footer pages inside pages.
 
 =head1 METHODS
 
-L<Muster::Hook::HeadFoot> inherits all methods from L<Muster::Hook::Directives>.
-
 =head2 register
 
 Do some intialization.
@@ -58,12 +56,6 @@ sub process {
     my $leaf = $args{leaf};
     my $phase = $args{phase};
 
-    # if this isn't a page, we can't include anything in it!
-    if (!$leaf->is_page)
-    {
-        return $leaf;
-    }
-
     # Don't do header/footer inclusion in the scanning phase;
     # it would be too confusing.
     if ($phase eq $Muster::Hooks::PHASE_SCAN)
@@ -76,13 +68,17 @@ sub process {
     # Do a naive solution first: simply include the raw text of the other pages
     # into this page, IFF they are of the same filetype.
     # This ought to work in most cases, if I'm sticking to Markdown for the default page type.
+    # If this is a binary file, use the "html_from" value if it exists.
     my %affix = ();
     $affix{header}->{pagename} = $self->_find_affix_page(current_page=>$pagename,
         affix_page=>'_Header');
     $affix{footer}->{pagename} = $self->_find_affix_page(current_page=>$pagename,
         affix_page=>'_Footer');
 
-    my $this_filetype = $leaf->filetype;
+    my $this_filetype = ($leaf->is_binary
+        ? (defined $leaf->meta->{html_from} and $leaf->meta->{html_from}
+            ? $leaf->meta->{html_from} : $leaf->filetype)
+        : $leaf->filetype);
     foreach my $type (keys %affix)
     {
         my $page = $affix{$type}->{pagename};
@@ -93,12 +89,14 @@ sub process {
             {
                 my $new_leaf = Muster::LeafFile->new(
                     pagename=>$info->{pagename},
+                    pagesrcname=>$info->{pagesrcname},
                     parent_page=>$info->{parent_page},
                     filename=>$info->{filename},
                     filetype=>$info->{filetype},
-                    is_page=>$info->{is_page},
+                    is_binary=>$info->{is_binary},
                     extension=>$info->{extension},
-                    name=>$info->{name},
+                    bald_name=>$info->{bald_name},
+                    hairy_name=>$info->{hairy_name},
                     title=>$info->{title},
                     date=>$info->{date},
                     meta=>$info,
@@ -153,11 +151,20 @@ sub _find_affix_page {
         return $local_sp;
     }
 
+    # Next priority is a grand-local page.
+    # This is in the directory *above* this page (grandparent)
+    # and has two extra '_' at the front of it.
+    my $grandlocal_sp = $cp_info->{grandparent_page} . '/__' . $affix_page;
+    if ($self->{metadb}->page_exists($grandlocal_sp))
+    {
+        return $grandlocal_sp;
+    }
+
     my @bits = split('/', $current_page);
     my $found_page = '';
     do {
         my $cwd = join('/', @bits);
-        my $q = "SELECT page FROM pagefiles WHERE name = '$affix_page' AND parent_page IN (SELECT parent_page FROM pagefiles WHERE page = '$cwd');";
+        my $q = "SELECT page FROM pagefiles WHERE bald_name = '$affix_page' AND parent_page IN (SELECT parent_page FROM pagefiles WHERE page = '$cwd');";
         my $pages = $self->{metadb}->query($q);
         if ($pages)
         {
