@@ -1,5 +1,5 @@
 package Muster::Hook::Include;
-$Muster::Hook::Include::VERSION = '0.62';
+$Muster::Hook::Include::VERSION = '0.92';
 use Mojo::Base 'Muster::Hook::Directives';
 use Muster::LeafFile;
 use Muster::Hooks;
@@ -12,7 +12,7 @@ Muster::Hook::Include - Muster include-page directive
 
 =head1 VERSION
 
-version 0.62
+version 0.92
 
 =head1 DESCRIPTION
 
@@ -84,7 +84,11 @@ sub process {
     # Do a naive solution first: simply include the raw text of the other pages
     # into this page, IFF they are of the same filetype.
     # This ought to work in most cases, if I'm sticking to Markdown for the default page type.
-    my $this_filetype = $leaf->filetype;
+    # If this is a binary file, use the "html_from" value if it exists.
+    my $this_filetype = ($leaf->is_binary
+        ? (defined $leaf->meta->{html_from} and $leaf->meta->{html_from}
+            ? $leaf->meta->{html_from} : $leaf->filetype)
+        : $leaf->filetype);
     my @in_stuff = ();
     foreach my $page (@list)
     {
@@ -93,12 +97,14 @@ sub process {
         {
             my $new_leaf = Muster::LeafFile->new(
                 pagename=>$info->{pagename},
+                pagesrcname=>$info->{pagesrcname},
                 parent_page=>$info->{parent_page},
                 filename=>$info->{filename},
                 filetype=>$info->{filetype},
-                is_page=>$info->{is_page},
+                is_binary=>$info->{is_binary},
                 extension=>$info->{extension},
-                name=>$info->{name},
+                bald_name=>$info->{bald_name},
+                hairy_name=>$info->{hairy_name},
                 title=>$info->{title},
                 date=>$info->{date},
                 meta=>$info,
