@@ -1,5 +1,5 @@
 package Muster::LeafFile;
-$Muster::LeafFile::VERSION = '0.62';
+$Muster::LeafFile::VERSION = '0.92';
 #ABSTRACT: Muster::LeafFile - a file in a Muster content tree
 =head1 NAME
 
@@ -7,7 +7,7 @@ Muster::LeafFile - a file in a Muster content tree
 
 =head1 VERSION
 
-version 0.62
+version 0.92
 
 =head1 SYNOPSIS
 
@@ -26,22 +26,25 @@ File nodes represent files in a content tree.
 use Mojo::Base -base;
 
 use Carp;
-use Mojo::Util      'decode';
 use File::Basename 'basename';
 use File::stat;
 use POSIX qw(strftime);
 use YAML::Any;
 use Lingua::EN::Titlecase;
+use File::MimeInfo::Magic;
 
 has pagename    => '';
+has pagesrcname    => '';
 has parent_page => '';
-has is_page     => sub { shift->is_this_a_page };
-has name        => sub { shift->build_name };
+has is_binary   => sub { shift->is_this_a_binary };
+has bald_name    => sub { shift->build_bald_name };
+has hairy_name    => sub { shift->build_hairy_name };
 has title       => sub { shift->build_title };
 has filename   => sub { croak 'no filename given' };
 has filetype   => sub { shift->build_filetype };
 has extension  => sub { shift->build_ext };
 has pagename   => sub { shift->build_pagename };
+has pagesrcname   => sub { shift->build_pagesrcname };
 
 =head2 raw
 
@@ -129,8 +132,8 @@ sub build_title {
     # try to extract title
     return $self->meta->{title} if exists $self->{meta} and exists $self->meta->{title};
     return $1 if defined $self->html and $self->html =~ m|<h1>(.*?)</h1>|i;
-    return $self->name;
-}
+    return $self->bald_name;
+} # build_title
 
 =head2 reclassify
 
@@ -159,35 +162,49 @@ sub reclassify {
     return $self;
 }
 
-=head2 build_name
+=head2 build_bald_name
 
-Build the base name of the related page.
+Build the base name of the related page with no suffix.
+It is "bald" because the suffix has been chopped off.
 
 =cut
-sub build_name {
+sub build_bald_name {
     my $self = shift;
 
     # get last filename part
     my $base = basename($self->filename);
 
-    # if this is a page as opposed to a non-page, delete the suffix
-    if ($self->is_page)
-    {
-        # delete suffix
-        $base =~ s/\.\w+$//;
-    }
+    # delete the suffix
+    $base =~ s/\.\w+$//;
 
     return $base;
-}
+} # build_bald_name
 
-=head2 is_this_a_page
+=head2 build_hairy_name
 
-By default, it is not a page. Returns undef.
+Build the base name of the related page, including the suffix.
+It is "hairy" because the suffix has not been chopped off.
 
 =cut
-sub is_this_a_page {
+sub build_hairy_name {
     my $self = shift;
-    return undef;
+
+    # get last filename part
+    my $base = basename($self->filename);
+
+    return $base;
+} # build_hairy_name
+
+=head2 is_this_a_binary
+
+If we don't know what it is, assume it is a binary file.
+Returns undef if the file is NOT a binary file.
+This is so we can use "IS NULL" tests in SQL for it.
+
+=cut
+sub is_this_a_binary {
+    my $self = shift;
+    return 1;
 }
 
 =head2 build_pagename
@@ -199,7 +216,19 @@ sub build_pagename {
     my $self = shift;
 
     # build from parent_page, infix slash and name
-    return join '/' => grep {$_ ne ''} $self->parent_page, $self->name;
+    return join '/' => grep {$_ ne ''} $self->parent_page, $self->bald_name;
+}
+
+=head2 build_pagesrcname
+
+Create the pagesrcname from the filename.
+
+=cut
+sub build_pagesrcname {
+    my $self = shift;
+
+    # build from parent_page, infix slash and hairy name
+    return join '/' => grep {$_ ne ''} $self->parent_page, $self->hairy_name;
 }
 
 =head2 build_filetype
@@ -241,18 +270,33 @@ sub build_ext {
 
 =head2 build_raw
 
-The raw content of the file.
+The raw content of the page.
 
 =cut
 sub build_raw {
     my $self = shift;
 
-    # open file for decoded reading
+    # We only want to read text files;
+    # since there could be all sorts of binary files
+    # that don't have their own LeafFile module,
+    # it behooves us to check before opening it.
     my $fn = $self->filename;
-    open my $fh, '<:encoding(UTF-8)', $fn or croak "couldn't open $fn: $!";
+    my $mime_type = mimetype($fn);
+    my $content = '';
+    if ($mime_type =~ /text/)
+    {
+        # Open file for decoded reading
+        open my $fh, '<:encoding(UTF-8)', $fn or croak "couldn't open $fn: $!";
 
-    # slurp
-    return do { local $/; <$fh> };
+        # slurp
+        $content = do { local $/; <$fh> };
+
+        # Test if it is really UTF-8
+        # See: https://www.perlmonks.org/?node_id=669902
+        utf8::decode($content)
+            or carp "LeafFile::build_raw INVALID UTF-8 ($mime_type) ", $self->filename;
+    }
+    return $content;
 }
 
 =head2 build_meta
@@ -273,22 +317,37 @@ sub build_meta {
     # file permissions have been changed without changing the content.  A file
     # may store its creation-date in its meta-data, but that will depend on the
     # filetype of that particular file, so we can't deal with it here.
+    # Therefore the most relevant file "date" comes from the mtime.
 
     my $st = stat($self->filename);
     my $date = strftime('%Y-%m-%d %H:%M', localtime($st->mtime));
 
+    my $grandparent_page = '';
+    if ($self->pagename =~ m{^(.*)/[-\.\w]+/[-\.\w]+$}o)
+    {
+        $grandparent_page = $1;
+    }
+    else # top-level page
+    {
+        $grandparent_page = '';
+    }
+
     # There is always the default information
     # of pagename, filename etc.
     my $meta = {
         pagename=>$self->pagename,
+        pagesrcname=>$self->pagesrcname,
         parent_page=>$self->parent_page,
         filename=>$self->filename,
         filetype=>$self->filetype,
-        is_page=>$self->is_page,
+        is_binary=>$self->is_binary,
         extension=>$self->extension,
-        name=>$self->name,
+        bald_name=>$self->bald_name,
+        hairy_name=>$self->hairy_name,
         title=>$self->derive_title,
         date=>$date,
+        mtime=>$st->mtime,
+        grandparent_page=>$grandparent_page,
     };
 
     return $meta;
@@ -303,7 +362,7 @@ sub derive_title {
     my $self = shift;
 
     # get the title from the name of the file
-    my $name = $self->name;
+    my $name = $self->bald_name;
     $name =~ s/[_-]/ /g;
     my $tc = Lingua::EN::Titlecase->new($name);
     return $tc->title();
@@ -311,18 +370,18 @@ sub derive_title {
 
 =head2 build_html
 
-Create the default HTML for non-page.
+Create the default HTML for an unknown page.
 
 =cut
 sub build_html {
     my $self = shift;
     
-    my $link = $self->pagename();
+    my $me = $self->pagename();
     my $title = $self->derive_title();
     return <<EOT;
 <h1>$title</h1>
 <p>
-<a href="/$link">$link</a>
+$me
 </p>
 EOT
 
