@@ -1,5 +1,5 @@
 package Muster::LeafFile::gif;
-$Muster::LeafFile::gif::VERSION = '0.62';
+$Muster::LeafFile::gif::VERSION = '0.92';
 #ABSTRACT: Muster::LeafFile::gif - a GIF file in a Muster content tree
 =head1 NAME
 
@@ -7,7 +7,7 @@ Muster::LeafFile::gif - a GIF file in a Muster content tree
 
 =head1 VERSION
 
-version 0.62
+version 0.92
 
 =head1 DESCRIPTION
 
@@ -21,11 +21,10 @@ use Mojo::Base 'Muster::LeafFile::EXIF';
 use Carp;
 use Image::ExifTool qw(:Public);
 
-# this is not a page
-sub is_this_a_page {
+sub is_this_a_binary {
     my $self = shift;
 
-    return undef;
+    return 1;
 }
 
 1;
