@@ -1,5 +1,5 @@
 package Muster::LeafFile::mdwn;
-$Muster::LeafFile::mdwn::VERSION = '0.62';
+$Muster::LeafFile::mdwn::VERSION = '0.92';
 #ABSTRACT: Muster::LeafFile::mdwn - a Markdown file in a Muster content tree
 =head1 NAME
 
@@ -7,7 +7,7 @@ Muster::LeafFile::mdwn - a Markdown file in a Muster content tree
 
 =head1 VERSION
 
-version 0.62
+version 0.92
 
 =head1 DESCRIPTION
 
@@ -26,15 +26,15 @@ use Hash::Merge;
 # use a fast YAML
 use YAML::XS;
 
-=head2 is_this_a_page
+=head2 is_this_a_binary
 
-Test if this type of file creates a page or is just a file.
+Test if this type of file which just contains binary data.
 
 =cut
-sub is_this_a_page {
+sub is_this_a_binary {
     my $self = shift;
 
-    return 1;
+    return undef;
 }
 
 =head2 build_meta
@@ -211,8 +211,8 @@ sub build_html {
 
     my $content = $self->cooked();
     # if the output is going to be text, don't process it
-    if (defined $self->meta->{page_format}
-            and $self->meta->{page_format} eq 'txt')
+    if (defined $self->meta->{render_format}
+            and $self->meta->{render_format} eq 'txt')
     {
         return $content;
     }
