@@ -1,5 +1,5 @@
 package Muster::Scanner;
-$Muster::Scanner::VERSION = '0.62';
+$Muster::Scanner::VERSION = '0.92';
 #ABSTRACT: Muster::Scanner - updating meta-data about pages
 =head1 NAME
 
@@ -7,7 +7,7 @@ Muster::Scanner - updating meta-data about pages
 
 =head1 VERSION
 
-version 0.62
+version 0.92
 
 =head1 DESCRIPTION
 
@@ -75,12 +75,32 @@ sub scan_some_pagefiles {
     my $self = shift;
     my @files = @_;
 
+    my %the_pages = ();
     foreach my $file (@files)
     {
-        $self->_scan_one_pagefile($file);
+        my $leaf = $self->_scan_one_pagefile($file);
+        if (!defined $leaf)
+        {
+            # the page was not found, therefore it has been deleted
+            my $pagename = $file;
+            $pagename =~ s/\.\w+$//; # remove the extension to get the pagename
+            if ($self->{metadb}->delete_one_page($pagename))
+            {
+                warn "DELETED: $file\n";
+            }
+            else
+            {
+                warn "UNKNOWN: $file\n";
+            }
+        }
+        else
+        {
+            $the_pages{$leaf->pagename} = $leaf->meta;
+        }
     }
+    $self->{metadb}->update_some_pages(%the_pages);
 
-    $self->{metadb}->update_derived_tables();
+    $self->{metadb}->update_derived_tables(pages=>{%the_pages});
 } # scan_some_pagefiles
 
 =head2 scan_all
@@ -105,8 +125,9 @@ These are private to the module.
 
 Scan a single pagefile; this expects the name of a file
 relative to the page_dir the file is in.
+This returns the leaf with all the relevant data in it.
 
-    $self->_scan_one_pagefile($filename);
+    my $leaf = $self->_scan_one_pagefile($filename);
 
 =cut
 
@@ -116,9 +137,7 @@ sub _scan_one_pagefile {
 
     return if !$filename;
 
-    my $pagename = $filename;
-    $pagename =~ s/\.\w+$//; # remove the extension to get the pagename
-    my $found_page;
+    my $found_leaf;
     foreach my $page_dir (@{$self->{page_dirs}})
     {
         my $finder = sub {
@@ -138,9 +157,9 @@ sub _scan_one_pagefile {
                     );
                     if ($leaf)
                     {
-                        if (!$found_page)
+                        if (!$found_leaf)
                         {
-                            $found_page = $leaf;
+                            $found_leaf = $leaf;
                         }
                     }
                 }
@@ -151,29 +170,7 @@ sub _scan_one_pagefile {
         find({wanted=>$finder, no_chdir=>1}, $page_dir);
     }
 
-    if (!defined $found_page)
-    {
-        # the page was not found, therefore it has been deleted
-        if ($self->{metadb}->delete_one_page($pagename))
-        {
-            warn "DELETED: $filename\n";
-        }
-        else
-        {
-            warn "UNKNOWN: $filename\n";
-        }
-    }
-    else
-    {
-        my $meta = $found_page->meta();
-        unless (defined $meta)
-        {
-            warn __PACKAGE__, " scan_one_pagefile meta for '$pagename' not found";
-            return;
-        }
-        # add the meta to the metadb
-        $self->{metadb}->update_one_page($pagename, %{$meta});
-    }
+    return $found_leaf;
 
 } # _scan_one_pagefile
 
